@@ -15,10 +15,14 @@ fi
 host_arch="${DEB_HOST_ARCH:-$(dpkg --print-architecture)}"
 
 if [ -z "${JDK_TO_TEST+x}" ]; then
-  JDK_TO_TEST=$(echo /usr/lib/jvm/java-18-openjdk-amd64 | sed "s/-[^-]*$/-$host_arch/")
+  JDK_TO_TEST=$(echo /usr/lib/jvm/java-21-openjdk-amd64 | sed "s/-[^-]*$/-$host_arch/")
 fi
 
-jtreg_version="$(dpkg-query -W jtreg6 | cut -f2)"
+if [ -z "${BOOTJDK_HOME+x}" ]; then
+  BOOTJDK_HOME=${JDK_TO_TEST}
+fi
+
+jtreg_version="$(dpkg-query -W jtreg7 | cut -f2)"
 
 # set additional jtreg options
 jt_options="${JTREG_OPTIONS:-}"
@@ -28,15 +32,20 @@ fi
 if dpkg --compare-versions ${jtreg_version} ge 4.2; then
   jt_options+=" -conc:auto"
 fi
-  
+
 # check java binary
 if [ ! -x "${JDK_TO_TEST}/bin/java" ]; then
-  echo "Error: '${JDK_TO_TEST}/bin/java' is not an executable." >&2
+  echo "Error: JDK_TO_TEST: '${JDK_TO_TEST}/bin/java' is not an executable." >&2
+  exit 1
+fi
+
+if [ ! -x "${BOOTJDK_HOME}/bin/java" ]; then
+  echo "Error: BOOTJDK_HOME: '${BOOTJDK_HOME}/bin/java' is not an executable." >&2
   exit 1
 fi
 
 # restrict the tests to a few archs (set from debian/rules)
-if ! echo "${host_arch}" | grep -qE "^($(echo amd64 i386 arm64 armhf ppc64 ppc64el s390x alpha ia64 powerpc ppc64 sh4 x32 armel mipsel mips64el riscv64 | tr ' ' '|'))$"; then
+if ! echo "${host_arch}" | grep -qE "^($(echo amd64 i386 arm64 armhf ppc64 ppc64el riscv64 s390x alpha ia64 powerpc ppc64 sh4 x32 armel mipsel mips64el riscv64 | tr ' ' '|'))$"; then
   echo "Error: ${host_arch} is not on the jtreg_archs list, ignoring it."
   exit 77
 fi
@@ -83,10 +92,11 @@ jtwork_dir="${AUTOPKGTEST_TMP}/${testsuite}/JTwork"
 output_dir="${AUTOPKGTEST_ARTIFACTS}/${testsuite}/"
 
 # retry tests with "fail" or "error" status at most 3 times
-for i in 0 1; do
+for i in 0 1 2; do
   # save each try under its own folder to preserve history
   report_path="${i}/JTreport"
   report_dir="${output_dir}/${report_path}"
+# see make/RunTests.gmk for a set of good options
   jtreg ${jt_options} \
     -verbose:summary \
     -automatic \
@@ -97,6 +107,11 @@ for i in 0 1; do
     -workDir:"${jtwork_dir}" \
     -reportDir:"${report_dir}" \
     -jdk:${JDK_TO_TEST} \
+    -vmoption:-Dtest.boot.jdk=${BOOTJDK_HOME} \
+    -vmoption:-Duser.home=${AUTOPKGTEST_TMP} \
+    -vmoption:-Djava.io.tmpdir=${AUTOPKGTEST_TMP} \
+    -vmoption:-XX:MaxRAMPercentage=25 \
+    -e:NSS_DEFAULT_DB_TYPE=sql \
     ${on_retry:-} $@ \
       && exit_code=0 || exit_code=$?
 
